"""Module that finds locations where buses exceed 50 km/h"""
import os
import pandas as pd
import warbusses.coors_functions as wcf

path = os.getcwd()
path_to_data = os.path.join(path, 'data')
df = pd.read_csv(os.path.join(path_to_data, 'speed.csv'), low_memory=False, index_col=0)
df = df[(df > 50).any(1)]

buses_above_50 = list(df.index)
buses_above_50 = list(set([x[:3] for x in buses_above_50]))

df2 = pd.read_csv(os.path.join(path_to_data, 'coordinates_speed.csv'),
                  low_memory=False, index_col=0)
locations_above_50 = []
p = list(enumerate(zip(df.values, df2.values)))

for i in p:
    for k in range(57):
        if i[1][0][k] > 50:
            locations_above_50.append(i[1][1][k])

numbers_loc = [0] * len(locations_above_50)

k = 0
for i in locations_above_50:
    for j in locations_above_50:
        try:
            if wcf.distance(i, j) < 0.5:
                numbers_loc[k] += 1
        except TypeError:
            continue
    k += 1
max_value = max(numbers_loc)
max_index = numbers_loc.index(max_value)

print(locations_above_50)
print(locations_above_50[max_index])



import requests
import os
import time

path = os.getcwd()
path_to_data = os.path.join(path, 'data')
hold_list = []


def extends():
    response = requests.get(
        "https://api.um.warszawa.pl/api/action/busestrams_get/?resource_id= f2e5503e-927d-4ad3-9500-4ab9e55deb59&apikey=eab89a39-4594-46b6-94cf-89eb29f728d5&type=1")
    resp = response.json()
    hold_list.extend(resp['result'])


if __name__ == "__main__":
    starttime = time.time()
    i = 0
    while (i < 60):
        extends()
        time.sleep(60.0 - ((time.time() - starttime) % 60.0))
        i = i + 1

    with open(os.path.join(path_to_data, 'bus_streams.txt'), 'w') as f:
        for item in hold_list:
            f.write("%s\n" % item)



import os
import pandas as pd
import ast

path = os.getcwd()
path_to_data = os.path.join(path, 'data')

with open(os.path.join(path_to_data, 'bus_streams.txt')) as fp:
    lines = fp.readlines()

lista = []

for line in lines:
    try:
        lista.append(ast.literal_eval(line))
    except ValueError:
        print("malformed string; skipping this line")
    except SyntaxError:
        print("looks like some encoding errors with this file...")

bus_info = []
for d in lista:
    bus_info.append([d["Lines"], d["VehicleNumber"], d["Time"], [(d["Lat"]), (d["Lon"])]])
array1 = [_ for _, __, col, val in bus_info]
array2 = [__ for _, __, col, val in bus_info]
arrays = [array1, array2]
tuples = list(zip(*arrays))

indeks = pd.MultiIndex.from_tuples(tuples, names=['Lines', 'VehicleNumber'])

df = pd.DataFrame([{col: val} for _ , __, col, val in bus_info], index=indeks)
df = df.groupby(level=[0, 1]).first()
df.columns = pd.to_datetime(df.columns)
df = df.reindex(sorted(df.columns), axis=1)
x1 = '2020-12-23 15:38:00'
x2 = '2020-12-23 16:42:00'
m1 = [i for i in df.columns if i > pd.to_datetime(x1)]
df = df[m1]
m2 = [i for i in df.columns if i < pd.to_datetime(x2)]
df = df[m2]

df.to_csv(os.path.join(path_to_data, 'bus_coordinates.csv'))



import pandas as pd


def transform_coors(szer_geo, dlug_geo):
    return str([szer_geo, dlug_geo])


def middle_coors(wsp1, wsp2):
    return [(a + b) / 2 for a, b in zip(list(get_coors(wsp1)), list(get_coors(wsp2)))]


def get_coors(wspx):
    wspx = wspx[1:-1]
    return float(wspx.split(',')[0]), float(wspx.split(',')[1])


def distance(wsp1, wsp2):
    """ takes as an input coordinates in a form str([52.2296756, 21.0122287])
    and returns distance between them in km."""
    from math import sin, cos, sqrt, atan2, radians
    wsp11, wsp12 = get_coors(wsp1)
    wsp21, wsp22 = get_coors(wsp2)
    # approximate radius of earth in km
    R = 6373.0

    lat1 = radians(float(wsp11))
    lon1 = radians(float(wsp12))
    lat2 = radians(float(wsp21))
    lon2 = radians(float(wsp22))

    dlon = lon2 - lon1
    dlat = lat2 - lat1

    a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))

    return R * c


def diff_between_dates(date1, date2):
    """ takes as an input dates in a form '23/12/2020  15:38:01'
    and returns difference between them in hours"""
    difference = abs(pd.to_datetime(date2) - pd.to_datetime(date1))
    return difference.seconds / 3600



import pandas as pd
import sys
import os
from webscraping_bus_stops import download_bus_schedule
import warbusses.coors_functions as wcf
import warbusses.bus_stop_functions as wbf

path = os.getcwd()
path_to_data = os.path.join(path, 'data')

df1 = pd.read_csv(os.path.join(path_to_data, 'bus_coordinates.csv'), low_memory=False)
df1[['Lines', 'VehicleNumber']] = df1[['Lines', 'VehicleNumber']].astype(str)
df2 = pd.read_csv(os.path.join(path_to_data, 'bus_stops.csv'), low_memory=False, encoding='utf-8')
df2["nazwa_zespolu"] = df2["nazwa_zespolu"].str.lower()

if __name__ == '__main__':
    bus_number = '212'
    # bus_number = input("Wpisz numer autobusu którego punktualność chcesz sprawdzić")
    vehicles = list(df1.loc[df1['Lines'] == bus_number, 'VehicleNumber'])
    # vehicle_number = input(f"Wybierz vehicle number z listy {vehicles}")
    vehicle_number = '9680'

    if bus_number not in df1['Lines'].values or vehicle_number not in vehicles:
        print('this bus is not in the data')
        sys.exit()

    row = df1.loc[df1['Lines'] == bus_number]
    row = row.loc[row['VehicleNumber'] == vehicle_number].dropna(axis=1)
    row = row.drop(['Lines', 'VehicleNumber'], axis=1)
    row = row.iloc[0]
    if row.empty:
        print('no coordinates for this line')
        sys.exit()

    bus_stops_two_directions, bus_stops = download_bus_schedule(bus_number)
    # Leave only those bus stops which are in bus_stop data.
    for lists in bus_stops_two_directions:
        for bus_stop in lists:
            if bus_stop not in df2["nazwa_zespolu"].values:
                lists.remove(bus_stop)
    for bus_stop in bus_stops:
        if bus_stop not in df2["nazwa_zespolu"].values:
            bus_stops.remove(bus_stop)

    df3 = df2.loc[df2['nazwa_zespolu'].isin(bus_stops)][['szer_geo', 'dlug_geo', 'nazwa_zespolu']]

    df3['coordinates'] = df3.apply(lambda x: wcf.transform_coors(x['szer_geo'], x['dlug_geo']),
                                   axis=1)

    df3['distance_from_bus_initial'] = df3.apply(lambda x: wcf.distance(x['coordinates'], row[0]),
                                                 axis=1)
    minimum_initial = df3['distance_from_bus_initial'].min()
    bus_stop_initial = df3[df3['distance_from_bus_initial'] == minimum_initial]['nazwa_zespolu'].iloc[0]

    direction, opposite_direction = wbf.check_direction(bus_stop_initial, bus_stops_two_directions, row, 3, df3)

    df3['distance_from_bus_final'] = df3.apply(lambda x: wcf.distance(x['coordinates'], row[-1]),
                                               axis=1)
    minimum_final = df3['distance_from_bus_final'].min()
    bus_stop_final = df3[df3['distance_from_bus_final'] == minimum_final]['nazwa_zespolu'].iloc[0]
    print(bus_stop_initial)
    print(bus_stop_final)
    times = []
    previous_bus_stop_time = row[row == row[0]].index[0]

    if bus_stop_final in direction[direction.index(bus_stop_initial):]:
        for bus_stop in direction[direction.index(bus_stop_initial):direction.index(bus_stop_final)]:
            times.append(wbf.get_time_for_bus_stop(bus_stop, previous_bus_stop_time, row, df3))
            previous_bus_stop_time = times[-1]
    else:
        for bus_stop in direction[direction.index(bus_stop_initial):]:
            print(bus_stop)
            times.append(wbf.get_time_for_bus_stop(bus_stop, previous_bus_stop_time, row, df3))
            previous_bus_stop_time = times[-1]
        for bus_stop in opposite_direction[:direction.index(bus_stop_final)]:
            print(bus_stop)
            times.append(wbf.get_time_for_bus_stop(bus_stop, previous_bus_stop_time, row, df3))
            previous_bus_stop_time = times[-1]

print(direction[direction.index(bus_stop_initial):])
print(opposite_direction[:direction.index(bus_stop_final)])
print(times)


from warbusses import coors_functions as cf


def check_direction(bus_stop_initial, bus_stops_two_directions, row, n, df3):
    distances_list = [0, 0]
    checking_row = row[1:n]
    for k, direction in enumerate(bus_stops_two_directions):
        distances = 0
        if bus_stop_initial not in direction:
            distances_list[k] = 1000
            continue
        else:
            index_bus_stop_initial = direction.index(bus_stop_initial)
            try:
                next_stop = direction[index_bus_stop_initial + 1]
            except IndexError:
                distances_list[k] = 1000
                continue
            next_stop_coordinates = df3[df3['nazwa_zespolu'] == next_stop]['coordinates'].iloc[0]
            for coordinate in checking_row:
                distances += cf.distance(next_stop_coordinates, coordinate)
            distances_list[k] = distances

    if distances_list[0] < distances_list[1]:
        return bus_stops_two_directions[0], bus_stops_two_directions[1]
    else:
        return bus_stops_two_directions[1], bus_stops_two_directions[0]


def get_time_for_bus_stop(bus_stop, previous_bus_stop_time, row, df3):
    bus_stop_coordinate = df3[df3['nazwa_zespolu'] == bus_stop]['coordinates'].iloc[0]
    coor = row[previous_bus_stop_time]
    row = row[previous_bus_stop_time:]
    minimum = cf.distance(bus_stop_coordinate, row[0])
    if row.size > 6:
        for coordinate in row[0:6]:
            if cf.distance(bus_stop_coordinate, coordinate) < minimum:
                minimum = cf.distance(bus_stop_coordinate, coordinate)
                coor = coordinate
        return row[row == coor].index[0]
    else:
        for coordinate in row[0:row.size]:
            if cf.distance(bus_stop_coordinate, coordinate) < minimum:
                minimum = cf.distance(bus_stop_coordinate, coordinate)
                coor = coordinate
        return row[row == coor].index[0]



"""Module that downloads data via API"""
import os
import json
import requests

path = os.getcwd()
path_to_data = os.path.join(path, 'data')


def extends():
    """Function that downloads bus stop data"""
    response = requests.get(
        '''https://api.um.warszawa.pl/api/action/dbstore_get?id=
            ab75c33d-3a26-4342-b36a-6e5fef0a3ac3&apikey=eab89a39-4594-46b6-94cf-89eb29f728d5''')
    resp = response.json()
    with open(os.path.join(path_to_data, 'bus_stops.txt'), 'w', encoding='utf-8') as outfile:
        json.dump(resp, outfile)


if __name__ == "__main__":
    extends()



import pandas as pd
import os
import json

path = os.getcwd()
path_to_data = os.path.join(path, 'data')

with open(os.path.join(path_to_data, 'bus_stops.txt')) as json_file:
    data = json.loads(json_file.read())

lista = data['result']

zespol, slupek, nazwa_zespolu, id_ulicy, szer_geo, dlug_geo, kierunek = [], [], [], [], [], [], []

for d in lista:
    slownik = d['values']
    for i in slownik:
        if 'zespol' in i.values():
            zespol.append(i['value'])
        elif 'slupek' in i.values():
            slupek.append(i['value'])
        elif 'nazwa_zespolu' in i.values():
            nazwa_zespolu.append(i['value'])
        elif 'id_ulicy' in i.values():
            id_ulicy.append(i['value'])
        elif 'szer_geo' in i.values():
            szer_geo.append(i['value'])
        elif 'dlug_geo' in i.values():
            dlug_geo.append(i['value'])
        elif 'kierunek' in i.values():
            kierunek.append(i['value'])
        else:
            continue

df = pd.DataFrame(list(zip(zespol, slupek, nazwa_zespolu, id_ulicy, szer_geo, dlug_geo, kierunek)),
                  columns=['zespol', 'slupek', 'nazwa_zespolu', 'id_ulicy', 'szer_geo', 'dlug_geo', 'kierunek'])

# potrzebujemy wlasciwie tylko zespol, slupek i wspolrzedne
df = df[['zespol', 'slupek', 'nazwa_zespolu', 'szer_geo', 'dlug_geo']]

df.to_csv(os.path.join(path_to_data, 'bus_stops.csv'), encoding='utf-8')



import setuptools

with open('readme.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='warbusses',
    version='0.4',
    description='Useful info about warsaw busses',
    long_description=long_description,
    packages=['warbusses'],
    classifiers=[
        'Programming Language :: Python :: 3'
    ],
    url='https://github.com/szczor/warbusses'

)



import pandas as pd
import os
import warbusses.coors_functions as wcf

path = os.getcwd()
path_to_data = os.path.join(path, 'data')

df = pd.read_csv(os.path.join(path_to_data, 'bus_coordinates.csv'), low_memory=False)
df[['Lines', 'VehicleNumber']] = df[['Lines', 'VehicleNumber']].astype(str)
df['linevehic'] = df[['Lines', 'VehicleNumber']].apply(lambda x: ''.join(x), axis=1)
df = df.drop(['Lines', 'VehicleNumber'], axis=1)
df = df.set_index('linevehic')

dfn = df.stack().reset_index()
dfn.columns = ['idx', 'time', 'coor']
dfn.sort_values(['idx', 'time'], inplace=True)

n = 2
lista_series, lista_coord = [], []

for i in range(57):
    res_dict, coord_dict = dict(), dict()
    for idx, group in dfn.groupby('idx'):
        try:
            dates = group.iloc[(n - 2):n, 1].tolist()
            coors = group.iloc[(n - 2):n, 2].tolist()

            # calculate use function
            dist_diff = wcf.distance(*coors)
            dates_diff = wcf.diff_between_dates(*dates)

            result = dist_diff / dates_diff
            res_dict[idx] = result
            coord_dict[idx] = wcf.middle_coors(*coors)
        except TypeError:
            print('nie ma danych')

    obj_res = pd.Series(res_dict)
    coord_dict = pd.Series(coord_dict)
    lista_series.append(obj_res)
    lista_coord.append(coord_dict)
    n = n + 1
df_speed = pd.concat(lista_series, axis=1)
df_coord = pd.concat(lista_coord, axis=1)
df_speed.to_csv(os.path.join(path_to_data, 'speed.csv'))
df_coord = df_coord.to_csv(os.path.join(path_to_data, 'coordinates_speed.csv'))



import requests
from bs4 import BeautifulSoup


def download_bus_schedule(bus_number):
    URL = "http://www.m2.rozkladzik.pl/warszawa/rozklad_jazdy.html?l=" + bus_number
    r = requests.get(URL)
    soup = BeautifulSoup(r.content,
                         'html5lib')

    bus_stops_1 = []
    bus_stops_2 = []

    directions = soup.find_all("ul", {"class": "holo-list"})

    for stop in directions[0].find_all("a"):
        if stop.text.lower().strip() not in bus_stops_1:
            bus_stops_1.append(stop.text.lower().strip())

    for stop in directions[1].find_all("a"):
        if stop.text.lower().strip() not in bus_stops_2:
            bus_stops_2.append(stop.text.lower().strip())

    bus_stops_two_directions = [bus_stops_1, bus_stops_2]

    return bus_stops_two_directions, sorted(bus_stops_two_directions[0] + list(set(bus_stops_two_directions[1])
                                                                        - set(bus_stops_two_directions[0])))